const defaultState = {
  loading: false,
  show: {
    seasons: [],
  },
};

export default function showStore(state = defaultState, action) {
  switch (action.type) {
    case 'SHOW_FETCH_DETAILS_PENDING':
      return Object.assign({}, state, {
        loading: true,
      })
    case 'SHOW_FETCH_DETAILS_FULFILLED':
      return Object.assign({}, state, {
        show: sortEpisodes(action.payload.response.data),
        loading: false,
      })
    case 'SHOW_UPDATE_STORE_WISHLIST':
      return Object.assign({}, state, {
        show: updateShowStoreWishlist(Object.assign({}, state.show), action.payload),
      })
    case 'EPISODE_GET_DETAILS':
      return Object.assign({}, state, {
        show: updateEpisode(Object.assign({}, state.show), true, action.payload),
      })
    case 'EPISODE_GET_DETAILS_FULFILLED':
      return Object.assign({}, state, {
        show: updateEpisode(Object.assign({}, state.show), false, action.payload.response.data),
      })
    case 'EPISODE_SUBTITLES_UPDATE_PENDING':
      return Object.assign({}, state, {
        show: updateEpisodeSubtitles(Object.assign({}, state.show), action.payload.main.season, action.payload.main.episode, true),
      })
    case 'EPISODE_SUBTITLES_UPDATE_FULFILLED':
      console.log("payload :", action.payload);
      return Object.assign({}, state, {
        show: updateEpisodeSubtitles(Object.assign({}, state.show), action.payload.main.season, action.payload.main.episode, false, action.payload.response.data),
      })
    default:
      return state
  }
}

function updateEpisode(show, fetching, data = null) {
  // Error handling for PouuleT
  if (data === null) {
    for (let seasonIndex of Object.keys(show.seasons)) {
      for (let episodeIndex of Object.keys(show.seasons[seasonIndex].episodes)) {
        show.seasons[seasonIndex].episodes[episodeIndex].fetching = false;
      }
    }
    return show
  }

  let seasonIndex = show.seasons.map((el) => el.season).indexOf(data.season.toString());
  let episodeIndex = show.seasons[seasonIndex].episodes.map((el) => el.episode).indexOf(data.episode);
  if ('imdb_id' in data) {
    show.seasons[seasonIndex].episodes[episodeIndex] = data;
  }
  show.seasons[seasonIndex].episodes[episodeIndex].fetching = fetching;
  return show
}

function updateEpisodeSubtitles(show, season, episode, fetching, data = null) {
  let seasonIndex = show.seasons.map((el) => el.season).indexOf(season.toString());
  let episodeIndex = show.seasons[seasonIndex].episodes.map((el) => el.episode).indexOf(episode);

  if (data) {
    show.seasons[seasonIndex].episodes[episodeIndex].subtitles = data;
  }
  show.seasons[seasonIndex].episodes[episodeIndex].fetchingSubtitles = fetching;
  return show
}

function sortEpisodes(show) {
  let episodes = show.episodes;
  delete show["episodes"];

  if (episodes.length == 0) {
    return show;
  }

  // Extract the seasons
  let seasons = {};
  for (let ep of episodes) {
    ep.fetching = false;
    if (!seasons[ep.season]) {
      seasons[ep.season] = { episodes: [] };
    }
    seasons[ep.season].episodes.push(ep);
  }

  if (seasons.length === 0) {
    return show;
  }

  // Put all the season in an array
  let sortedSeasons = [];
  for (let season of Object.keys(seasons)) {
    let seasonEpisodes = seasons[season].episodes;
    // Order the episodes in each season
    seasonEpisodes.sort((a,b) => (a.episode - b.episode))
    // Add the season in the list
    sortedSeasons.push({
      season: season,
      episodes: seasonEpisodes,
    })
  }

  // Order the seasons
  for (let i=0; i<sortedSeasons.length; i++) {
    sortedSeasons.sort((a,b) => (a.season - b.season))
  }

  show.seasons = sortedSeasons;

  return show;
}

// Update the store containing the current detailed show
function updateShowStoreWishlist(show, payload) {
  if (show.seasons.length === 0) {
    return show;
  }
  let season = payload.season;
  let episode = payload.episode;
  if (payload.wishlisted) {
    if (season === null) {
      season = 0;
    }
    if (episode === null) {
      episode = 0;
    }
  }
  show.tracked_season = season;
  show.tracked_episode = episode;
  return show
}
