const defaultState = {
  loading: false,
  shows: [],
  filter: "",
  perPage: 30,
  selectedImdbId: "",
  getDetails: false,
  lastShowsFetchUrl: "",
  exploreOptions: {},
};

export default function showsStore(state = defaultState, action) {
  switch (action.type) {
    case 'SHOW_LIST_FETCH_PENDING':
      return Object.assign({}, state, {
        loading: true,
      })
    case 'SHOW_LIST_FETCH_FULFILLED':
      let selectedImdbId = "";
      // Select the first show
      console.log("Hey", action.payload);
      if (action.payload.response.data.length > 0) {
        selectedImdbId = action.payload.response.data[0].imdb_id;
      }
      return Object.assign({}, state, {
        shows: action.payload.response.data,
        selectedImdbId: selectedImdbId,
        filter: defaultState.filter,
        loading: false,
      })
    case 'SHOW_GET_DETAILS_PENDING':
      return Object.assign({}, state, {
        getDetails: true,
      })
    case 'SHOW_GET_DETAILS_FULFILLED':
      return Object.assign({}, state, {
        shows: updateShowDetails(state.shows.slice(), action.payload.response.data),
        getDetails: false,
      })
    case 'EXPLORE_SHOWS_PENDING':
      return Object.assign({}, state, {
        loading: true,
      })
    case 'EXPLORE_SHOWS_FULFILLED':
      return Object.assign({}, state, {
        shows: action.payload.response.data,
        loading: false,
      })
    case 'SHOW_GET_EXPLORE_OPTIONS_FULFILLED':
      return Object.assign({}, state, {
        exploreOptions: action.payload.response.data,
      })
    case 'SHOW_UPDATE_STORE_WISHLIST':
      return Object.assign({}, state, {
        shows: updateShowsStoreWishlist(state.shows.slice(), action.payload),
      })
    case 'UPDATE_LAST_SHOWS_FETCH_URL':
      return Object.assign({}, state, {
        lastShowsFetchUrl: action.payload.url,
      })
    case 'SELECT_SHOW':
      // Don't select the show if we're fetching another show's details
      if (state.fetchingDetails) {
        return state
      }

      return Object.assign({}, state, {
        selectedImdbId: action.imdbId,
      })
    default:
      return state
  }
}

// Update the store containing all the shows
function updateShowsStoreWishlist(shows, payload) {
  if (shows.length === 0) {
    return shows;
  }

  let index = shows.map((el) => el.imdb_id).indexOf(payload.imdbId);
  let season = payload.season;
  let episode = payload.episode;
  if (payload.wishlisted) {
    if (season === null) {
      season = 0;
    }
    if (episode === null) {
      episode = 0;
    }
  }
  shows[index].tracked_season = season;
  shows[index].tracked_episode = episode;
  return shows
}

function updateShowDetails(shows, data) {
  let index = shows.map((el) => el.imdb_id).indexOf(data.imdb_id);
  shows[index] = data;
  return shows
}
